/*
 * Rapid Paste Plugin for Diodon
 * Copyright (C) 2013 Nathaniel Nutter <iam@nnutter.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Diodon.Plugins.RapidPaste {
    public class RapidPastePlugin : Peas.ExtensionBase, Peas.Activatable, PeasGtk.Configurable {
        private KeybindingManager keybinding_manager;
        private Controller controller;
        private ConfigurationModel configuration_model;
        private Settings _settings;
        private string accelerator;
        private int history_offset = 0;

        public Object object { get; construct; }

        public RapidPastePlugin() {
            Object();
        }

        public void activate() {
            controller = object as Controller;

            var settings = get_settings();

            if (accelerator == null) {
                accelerator = settings.get_string("accelerator");
                bind_accelerator(accelerator);
            }

            settings.changed.connect((key) => {
                var new_accelerator = settings.get_string("accelerator");
                if (new_accelerator != accelerator) {
                    bind_accelerator(new_accelerator);
                }
            });

            controller.on_add_item.connect(reset_history_offset);
            configuration_model = controller.get_configuration();
        }

        public void deactivate() {
            keybinding_manager = controller.get_keybinding_manager();
            keybinding_manager.unbind(accelerator);

            controller.on_add_item.disconnect(reset_history_offset);
        }

        public void update_state() {
        }

        private Settings get_settings() {
            if (_settings == null) {
                _settings = new Settings("com.diodon.plugins.rapid-paste");
            }
            return _settings;
        }

        private void reset_history_offset() {
            history_offset = 0;
        }

        private void rapid() {
            var items = controller.get_items();
            int item_index = (items.size - 1) - history_offset;
            var item = items.get(item_index);

            ulong delay = 100;
            controller.select_item_with_delay(item, delay);
            if(!configuration_model.instant_paste) {
                controller.execute_paste_with_delay(item, delay);
            }

            if (history_offset < (items.size - 1)) {
                history_offset++;
            }
        }

        private void bind_accelerator(string new_accelerator) {
            controller = object as Controller;
            keybinding_manager = controller.get_keybinding_manager();
            if (accelerator != null) {
                keybinding_manager.unbind(accelerator);
            }
            keybinding_manager.bind(new_accelerator, rapid);
            accelerator = new_accelerator;
        }

        public Gtk.Widget create_configure_widget() {
            var settings = get_settings();
            accelerator = settings.get_string("accelerator");

            var box = new Gtk.Box(Gtk.Orientation.HORIZONTAL, 5);
            var label = new Gtk.Label("Rapid Paste Key:");
            var entry = new Gtk.Entry();
            entry.set_text(accelerator);
            entry.focus_out_event.connect(() => {
                string text = entry.get_text();
                if (text != null) {
                    settings.set_string("accelerator", text);
                }
                return true;
            });
            box.pack_start(label);
            box.pack_start(entry);
            label.show();
            return box;
        }
    }
}

[ModuleInit]
public void peas_register_types (GLib.TypeModule module) {
  var objmodule = module as Peas.ObjectModule;
  objmodule.register_extension_type (typeof (Peas.Activatable),
                                     typeof (Diodon.Plugins.RapidPaste.RapidPastePlugin));
  objmodule.register_extension_type (typeof (PeasGtk.Configurable),
                                     typeof (Diodon.Plugins.RapidPaste.RapidPastePlugin));
}

